<?php
namespace metagen_cs_ecs;

function supported_tokens()
{
  return [
    'serializable',
    'ecs_gen_component',
    'ecs_gen_authoring',
    'ecs_tag',
  ];
}

function get_twig_inc()
{
  return __DIR__ ."/../tpl";
}

function get_gen_root()
{
  global $GAME_ROOT;
  return "$GAME_ROOT/unity/Assets/Scripts/autogen/ecs";
}

function generate_ecs_components(\mtgMetaInfo $meta)
{
  global $GAME_ROOT;

  $gen_root = get_gen_root();
  $gen_components = new \mtgMetaInfo();
  $gen_authorings = new \mtgMetaInfo();
  $lock_file = "$gen_root/ecs.lock";

  if(!\taskman\need_to_regen($lock_file, get_meta_units_files($meta)))
  {
    return;
  }
  \taskman\ensure_rm($gen_root);
  \taskman\ensure_write($lock_file, "");
  touch($lock_file);

  foreach ($meta->getUnits() as $unit)
  {
    if(!($unit->object instanceof \mtgUserType))
      continue;
    if($unit->object->hasToken('ecs_gen_component'))
      $gen_components->addUnit($unit);
    if($unit->object->hasToken('ecs_gen_authoring'))
      $gen_authorings->addUnit($unit);
  }
  
  $twig = \metagen_cs\get_twig(array(get_twig_inc()));
  _add_twig_support($twig);
  $twig->setCache($GAME_ROOT . "/build/twig/metagen_cs_ecs");

  foreach ($gen_components->getUnits() as $unit)
  {
    $unit_name = $unit->object->getName();
    $filePath = str_replace(".", "/", $unit_name);
    $file = "$gen_root/$filePath.cs";
    $name_components = explode('.', $unit_name);
    $class_name = end($name_components);
    if($class_name != $unit_name)
      $namespace = str_replace(".$class_name", "", $unit_name);
    else
      $namespace = "";

    // TODO: Костыль. Убрать, когда будет поддержка namespace в bhl_bind
    if(empty($namespace))
      $namespace = "ecs";

    \taskman\ensure_write_if_differs(
      $file,
      $twig->render('codegen_ecs_component.twig', [
        'namespace' => $namespace,
        'class_name' => $class_name,
        'obj' => $unit->object,
        'imports' => [
          'System.Collections',
          'System.Collections.Generic',
          'System',
          'BitGames',
          'BitGames.Autogen',
          'UnityEngine',
          'bhl',
          'Leopotam.Ecs'
        ],
      ])
    );
  }

  foreach ($gen_authorings->getUnits() as $unit)
  {
    $unit_name = $unit->object->getName();
    $filePath = str_replace(".", "/", $unit_name);
    $file = "$gen_root/{$filePath}AuthoringComponent.cs";
    $name_components = explode('.', $unit_name);
    $class_name = end($name_components);
    if($class_name != $unit_name)
      $namespace = str_replace(".$class_name", "", $unit_name);
    else
      $namespace = "";

    // TODO: Костыль. Убрать, когда будет поддержка namespace в bhl_bind
    if(empty($namespace))
      $namespace = "ecs";

    if($unit->object->hasToken('ecs_gen_authoring'))
    {
      \taskman\ensure_write_if_differs(
        $file,
        $twig->render('codegen_ecs_authoring.twig', [
          'namespace' => $namespace,
          'class_name' => $class_name,
          'obj' => $unit->object
        ])
      );
    }
  }
}

function get_meta_units_files(\mtgMetaInfo $meta)
{
  $files = array();
  foreach($meta->getUnits() as $u)
    $files[$u->file] = true;
  return array_keys($files);
}

function _add_twig_support(\Twig\Environment $twig)
{
  $twig->addFilter(new \Twig\TwigFilter('ecs_type', 
    function($type)
    {
      if(($type instanceof \mtgUserType) && $type->hasToken("bhl_native_class"))
        return $type->getToken("bhl_native_class");
      return \metagen_cs\cs_type($type);
    }
  ));
}